Package: missPLS
Type: Package
Title: Methods and Reproducible Workflows for Partial Least Squares
        with Missing Data
Version: 0.2.1
Date: 2026-04-07
Depends: R (>= 4.1.0)
Imports: mice, plsRglm, stats, utils, VIM
Suggests: bcv, knitr, mlbench, plsdof, rmarkdown, testthat (>= 3.0.0)
Authors@R: c(
    person("Titin Agustin", "Nengsih", role = "aut"),
    person("Frederic", "Bertrand", role = c("aut", "cre"), email = "frederic.bertrand@lecnam.net"),
    person("Myriam", "Maumy-Bertrand", role = "aut"))
Author: Titin Agustin Nengsih [aut],
  Frederic Bertrand [aut, cre],
  Myriam Maumy-Bertrand [aut]
Maintainer: Frederic Bertrand <frederic.bertrand@lecnam.net>
Description: Methods-first tooling for reproducing and extending the
    partial least squares regression studies on incomplete data described in
    Nengsih et al. (2019) <doi:10.1515/sagmb-2018-0059>. The package
    provides simulation helpers, missingness generators, imputation wrappers,
    component-selection utilities, real-data diagnostics, and reproducible
    study orchestration for Nonlinear Iterative Partial Least Squares (NIPALS)-Partial 
    Least Squares (PLS) workflows.
License: GPL-3
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://fbertran.github.io/missPLS/,
        https://github.com/fbertran/missPLS
BugReports: https://github.com/fbertran/missPLS/issues
NeedsCompilation: no
Packaged: 2026-04-29 20:57:19 UTC; bertran7
Repository: CRAN
Date/Publication: 2026-04-30 07:50:09 UTC
Built: R 4.6.0; ; 2026-04-30 11:26:52 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,misspls_dataset)
S3method(print,misspls_imputation)
export(add_missingness)
export(diagnose_real_data)
export(impute_pls_data)
export(run_real_data_study)
export(run_simulation_study)
export(select_ncomp)
export(simulate_pls_data)
export(summarize_simulation_study)

#' Diagnose a real dataset
#'
#' Compute correlation summaries and VIF-style diagnostics for a packaged
#' real dataset.
#'
#' @param dataset A packaged dataset name or `misspls_dataset` object.
#' @param cor_threshold Absolute-correlation threshold used when reporting
#'   predictor pairs and predictor-response associations.
#'
#' @return A list with correlation and VIF summaries.
#' @export
#'
#' @examples
#' diag_bromhexine <- diagnose_real_data("bromhexine")
#' names(diag_bromhexine)
diagnose_real_data <- function(dataset, cor_threshold = 0.7) {
  dataset <- .coerce_dataset(dataset)
  x <- dataset$x
  y <- dataset$y

  cor_x <- stats::cor(x)
  diag(cor_x) <- NA_real_
  upper_idx <- which(upper.tri(cor_x) & abs(cor_x) >= cor_threshold, arr.ind = TRUE)
  predictor_pairs <- if (nrow(upper_idx)) {
    data.frame(
      predictor_1 = colnames(x)[upper_idx[, 1L]],
      predictor_2 = colnames(x)[upper_idx[, 2L]],
      correlation = cor_x[upper_idx],
      stringsAsFactors = FALSE
    )
  } else {
    data.frame(
      predictor_1 = character(0),
      predictor_2 = character(0),
      correlation = numeric(0),
      stringsAsFactors = FALSE
    )
  }

  response_cor <- stats::cor(x, y)
  response_hits <- which(abs(response_cor) >= cor_threshold)
  response_pairs <- if (length(response_hits)) {
    data.frame(
      predictor = colnames(x)[response_hits],
      correlation = response_cor[response_hits],
      stringsAsFactors = FALSE
    )
  } else {
    data.frame(
      predictor = character(0),
      correlation = numeric(0),
      stringsAsFactors = FALSE
    )
  }

  vif_values <- .vif_values(x)
  vif_df <- data.frame(
    predictor = colnames(x),
    vif = vif_values,
    stringsAsFactors = FALSE
  )

  list(
    dataset = dataset$name,
    n = nrow(x),
    p = ncol(x),
    predictor_correlations = predictor_pairs,
    response_correlations = response_pairs,
    vif = vif_df,
    notes = if (nrow(x) <= ncol(x)) {
      "VIF values are undefined for n <= p and are returned as NA."
    } else {
      "VIF values were computed by regressing each predictor on the remaining predictors."
    }
  )
}

load_misspls <- function() {
  if (!"package:missPLS" %in% search()) {
    devtools::load_all(".", quiet = TRUE)
  }
}

make_output_dir <- function(name, root_dir) {
  if (missing(root_dir) || !nzchar(root_dir)) {
    stop("`root_dir` must be supplied for writing helpers.", call. = FALSE)
  }

  out_dir <- file.path(root_dir, "reproduction", name)
  dir.create(out_dir, recursive = TRUE, showWarnings = FALSE)
  out_dir
}

write_manifest <- function(output_dir, parameters) {
  param_file <- file.path(output_dir, "parameters.txt")
  writeLines(capture.output(str(parameters)), con = param_file)

  session_file <- file.path(output_dir, "session-info.txt")
  writeLines(capture.output(sessionInfo()), con = session_file)
}

write_discrepancy_report <- function(output_dir, label, current_summary, reference = NULL) {
  report_file <- file.path(output_dir, "discrepancy-report.txt")
  lines <- c(
    paste("Report:", label),
    paste("Generated:", format(Sys.time(), tz = "UTC", usetz = TRUE)),
    ""
  )

  if (is.null(reference)) {
    lines <- c(
      lines,
      "No encoded published reference table is bundled with the package.",
      "Manual comparison against the article or thesis tables is still required for this run.",
      "",
      "Current summary preview:",
      capture.output(utils::head(current_summary))
    )
  } else {
    lines <- c(
      lines,
      "A reference table was supplied by the caller.",
      "",
      "Current summary preview:",
      capture.output(utils::head(current_summary)),
      "",
      "Reference preview:",
      capture.output(utils::head(reference))
    )
  }

  writeLines(lines, con = report_file)
}

save_outputs <- function(output_dir, prefix, results, summary_df) {
  utils::write.csv(results, file.path(output_dir, paste0(prefix, "-results.csv")), row.names = FALSE)
  utils::write.csv(summary_df, file.path(output_dir, paste0(prefix, "-summary.csv")), row.names = FALSE)
  saveRDS(results, file.path(output_dir, paste0(prefix, "-results.rds")))
  saveRDS(summary_df, file.path(output_dir, paste0(prefix, "-summary.rds")))
}

write_match_plot <- function(output_dir, prefix, summary_df, title) {
  png(file.path(output_dir, paste0(prefix, "-match-rate.png")), width = 1400, height = 900, res = 150)
  on.exit(dev.off(), add = TRUE)

  plot_df <- summary_df[!is.na(summary_df$match_rate), , drop = FALSE]
  if (!nrow(plot_df)) {
    plot.new()
    title(main = paste(title, "(no non-missing match rates)"))
    return(invisible(NULL))
  }

  labels <- paste(plot_df$method, plot_df$criterion, sep = "\n")
  stats::barplot(
    height = plot_df$match_rate,
    names.arg = labels,
    las = 2,
    col = "steelblue",
    ylim = c(0, 1),
    main = title,
    ylab = "Match rate"
  )
}

#' Add missing values to a predictor matrix
#'
#' Create MCAR or MAR missingness on the predictor matrix `x`. Missingness is
#' generated column-wise so that each predictor receives approximately the same
#' missing-data proportion, matching the simulation strategy used in the
#' original work.
#'
#' @param x Predictor matrix or data frame.
#' @param y Numeric response vector.
#' @param mechanism Missingness mechanism: `"MCAR"` or `"MAR"`.
#' @param missing_prop Missing-data proportion as a fraction (`0.05`) or a
#'   percentage (`5`).
#' @param seed Optional random seed. If supplied, it is used only for this call.
#' @param mar_y_bias Proportion of missing values assigned to the upper
#'   half of the observed `y` values under the MAR mechanism.
#'
#' @return A list with components `x_incomplete`, `missing_mask`,
#'   `missing_prop`, `mechanism`, and `seed`.
#' @export
#'
#' @examples
#' sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 1)
#' miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
#' mean(is.na(miss$x_incomplete))
add_missingness <- function(x, y, mechanism = c("MCAR", "MAR"), missing_prop, seed = NULL,
                            mar_y_bias = 0.8) {
  mechanism <- match.arg(mechanism)
  seed <- .set_seed_if_supplied(seed)
  prop <- .normalize_missing_prop(missing_prop)
  x <- .as_numeric_matrix(x)
  y <- as.numeric(y)

  if (nrow(x) != length(y)) {
    stop("`x` and `y` must have compatible dimensions.", call. = FALSE)
  }
  if (mar_y_bias <= 0 || mar_y_bias >= 1) {
    stop("`mar_y_bias` must lie strictly between 0 and 1.", call. = FALSE)
  }

  n <- nrow(x)
  p <- ncol(x)
  n_missing_per_col <- max(1L, as.integer(round(n * prop)))
  if (n_missing_per_col >= n) {
    stop("`missing_prop` is too large for the current number of rows.", call. = FALSE)
  }

  x_incomplete <- x
  missing_mask <- matrix(FALSE, nrow = n, ncol = p, dimnames = dimnames(x))

  for (j in seq_len(p)) {
    if (mechanism == "MCAR") {
      rows <- sample.int(n, size = n_missing_per_col, replace = FALSE)
    } else {
      split_point <- stats::median(y, na.rm = TRUE)
      high_rows <- which(y > split_point)
      low_rows <- which(y <= split_point)
      n_high <- min(length(high_rows), max(0L, as.integer(round(n_missing_per_col * mar_y_bias))))
      n_low <- min(length(low_rows), n_missing_per_col - n_high)
      rows <- c(
        if (n_high) sample(high_rows, size = n_high, replace = FALSE) else integer(0),
        if (n_low) sample(low_rows, size = n_low, replace = FALSE) else integer(0)
      )
      if (length(rows) < n_missing_per_col) {
        remainder_pool <- setdiff(seq_len(n), rows)
        rows <- c(rows, sample(remainder_pool, size = n_missing_per_col - length(rows), replace = FALSE))
      }
    }

    x_incomplete[rows, j] <- NA_real_
    missing_mask[rows, j] <- TRUE
  }

  if (any(apply(is.na(x_incomplete), 1L, all))) {
    return(add_missingness(
      x = x,
      y = y,
      mechanism = mechanism,
      missing_prop = prop,
      seed = .seed_with_offset(seed, 1009L),
      mar_y_bias = mar_y_bias
    ))
  }

  list(
    x_incomplete = x_incomplete,
    missing_mask = missing_mask,
    missing_prop = prop,
    mechanism = mechanism,
    seed = .seed_column_value(seed)
  )
}

test_that("missingness generation is deterministic and avoids all-missing rows", {
  sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
  miss_1 <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
  miss_2 <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)

  expect_identical(miss_1$x_incomplete, miss_2$x_incomplete)
  expect_false(any(apply(is.na(miss_1$x_incomplete), 1L, all)))
  expect_gt(mean(is.na(miss_1$x_incomplete)), 0)
})

test_that("MAR missingness is deterministic", {
  sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
  miss_1 <- add_missingness(sim$x, sim$y, mechanism = "MAR", missing_prop = 10, seed = 3)
  miss_2 <- add_missingness(sim$x, sim$y, mechanism = "MAR", missing_prop = 10, seed = 3)

  expect_identical(miss_1$x_incomplete, miss_2$x_incomplete)
  expect_false(any(apply(is.na(miss_1$x_incomplete), 1L, all)))
})

test_that("simulation and missingness can rely on external RNG state", {
  set.seed(101)
  sim_1 <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2)
  set.seed(101)
  sim_2 <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2)

  expect_identical(sim_1$x, sim_2$x)
  expect_true(is.na(sim_1$seed))

  set.seed(202)
  miss_1 <- add_missingness(sim_1$x, sim_1$y, mechanism = "MCAR", missing_prop = 10)
  set.seed(202)
  miss_2 <- add_missingness(sim_1$x, sim_1$y, mechanism = "MCAR", missing_prop = 10)

  expect_identical(miss_1$x_incomplete, miss_2$x_incomplete)
  expect_true(is.na(miss_1$seed))
})

test_that("imputation wrappers return reproducible outputs", {
  sim <- simulate_pls_data(n = 25, p = 10, true_ncomp = 2, seed = 1)
  miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)

  imp_knn_1 <- impute_pls_data(miss$x_incomplete, method = "knn", seed = 3)
  imp_knn_2 <- impute_pls_data(miss$x_incomplete, method = "knn", seed = 3)
  expect_s3_class(imp_knn_1, "misspls_imputation")
  expect_identical(imp_knn_1$datasets[[1]], imp_knn_2$datasets[[1]])

  imp_svd <- impute_pls_data(miss$x_incomplete, method = "svd", seed = 4, svd_rank = 3, svd_maxiter = 50)
  expect_s3_class(imp_svd, "misspls_imputation")
  expect_false(anyNA(imp_svd$datasets[[1]]))

  imp_mice <- impute_pls_data(miss$x_incomplete, method = "mice", seed = 5, m = 2)
  expect_s3_class(imp_mice, "misspls_imputation")
  expect_length(imp_mice$datasets, 2L)
})

#!/usr/bin/env Rscript

source("tools/repro_helpers.R")
load_misspls()

reps <- as.integer(Sys.getenv("MISSPLS_REPS", "1000"))
output_root <- Sys.getenv("MISSPLS_OUTPUT_DIR", unset = "")
if (!nzchar(output_root)) {
  output_root <- tempdir()
}
output_dir <- make_output_dir("incomplete-data", root_dir = output_root)

results <- missPLS::run_simulation_study(
  dimensions = list(c(500L, 100L), c(500L, 20L), c(100L, 20L), c(80L, 25L),
                    c(60L, 33L), c(40L, 50L), c(20L, 100L)),
  true_ncomp = c(2L, 4L, 6L),
  missing_props = seq(5, 50, 5),
  mechanisms = c("MCAR", "MAR"),
  reps = reps,
  seed = 20250407L,
  max_ncomp = 8L,
  criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
  incomplete_methods = c("nipals_standard", "nipals_adaptative"),
  imputation_methods = c("mice", "knn", "svd"),
  folds = 10L
)

summary_df <- missPLS::summarize_simulation_study(results)
save_outputs(output_dir, "incomplete-data", results, summary_df)
write_manifest(output_dir, list(reps = reps, script = "run_incomplete_data_simulations.R"))
write_match_plot(output_dir, "incomplete-data", summary_df, "Incomplete-data simulation match rates")
write_discrepancy_report(output_dir, "Incomplete data simulations", summary_df)

#' Bromhexine dataset
#'
#' Bromhexine in pharmaceutical syrup used in the article and thesis.
#'
#' @format A `misspls_dataset` list with components:
#' \describe{
#'   \item{name}{Dataset name.}
#'   \item{x}{A numeric `23 x 64` predictor matrix.}
#'   \item{y}{A numeric response vector of length `23`.}
#'   \item{data}{A data frame with response `y` and predictors `x1` to `x64`.}
#'   \item{source}{A short source reference.}
#'   \item{preprocessing}{Dataset preprocessing notes.}
#'   \item{notes}{Additional study notes.}
#' }
#' @source Goicoechea and Olivieri (1999a), calibration and test files bundled in
#'   `extra_docs/pls_data`.
"bromhexine"

#' Tetracycline dataset
#'
#' Tetracycline in serum used in the article and thesis.
#'
#' @format A `misspls_dataset` list with components:
#' \describe{
#'   \item{name}{Dataset name.}
#'   \item{x}{A numeric `107 x 101` predictor matrix.}
#'   \item{y}{A numeric response vector of length `107`.}
#'   \item{data}{A data frame with response `y` and predictors `x1` to `x101`.}
#'   \item{source}{A short source reference.}
#'   \item{preprocessing}{Dataset preprocessing notes.}
#'   \item{notes}{Additional study notes.}
#' }
#' @source Goicoechea and Olivieri (1999b), calibration and test files bundled in
#'   `extra_docs/pls_data`.
"tetracycline"

#' Octane dataset
#'
#' Octane in gasoline from NIR data used in the article and thesis.
#'
#' @format A `misspls_dataset` list with components:
#' \describe{
#'   \item{name}{Dataset name.}
#'   \item{x}{A numeric `68 x 493` predictor matrix.}
#'   \item{y}{A numeric response vector of length `68`.}
#'   \item{data}{A data frame with response `y` and predictors `x1` to `x493`.}
#'   \item{source}{A short source reference.}
#'   \item{preprocessing}{Dataset preprocessing notes.}
#'   \item{notes}{Additional study notes.}
#' }
#' @source Goicoechea and Olivieri (2003), calibration and test files bundled in
#'   `extra_docs/pls_data`.
"octane"

#' Complete-case ozone dataset
#'
#' Los Angeles ozone pollution complete-case dataset used in the article and
#' thesis.
#'
#' @format A `misspls_dataset` list with components:
#' \describe{
#'   \item{name}{Dataset name.}
#'   \item{x}{A numeric `203 x 12` predictor matrix.}
#'   \item{y}{A numeric response vector of length `203`.}
#'   \item{data}{A data frame with response `y` and predictors `x1` to `x12`.}
#'   \item{source}{A short source reference.}
#'   \item{preprocessing}{Dataset preprocessing notes.}
#'   \item{notes}{Additional study notes.}
#' }
#' @source `mlbench::Ozone`, restricted to the `203` complete observations used
#'   in the published analysis.
"ozone_complete"

`%||%` <- function(x, y) {
  if (is.null(x)) {
    y
  } else {
    x
  }
}

.normalize_seed <- function(seed, allow_null = FALSE) {
  if (is.null(seed)) {
    if (allow_null) {
      return(NULL)
    }
    stop("`seed` must be a single non-missing integer-like value.", call. = FALSE)
  }

  if (length(seed) != 1L || is.na(seed)) {
    stop("`seed` must be a single non-missing integer-like value.", call. = FALSE)
  }

  as.integer(seed)
}

.seed_with_offset <- function(seed, offset) {
  base_seed <- .normalize_seed(seed, allow_null = TRUE)
  if (is.null(base_seed)) {
    return(NULL)
  }
  max_int <- .Machine$integer.max
  as.integer((base_seed + as.integer(offset)) %% max_int)
}

.set_seed_if_supplied <- function(seed) {
  seed <- .normalize_seed(seed, allow_null = TRUE)
  if (!is.null(seed)) {
    set.seed(seed)
  }
  seed
}

.seed_column_value <- function(seed) {
  seed <- .normalize_seed(seed, allow_null = TRUE)
  if (is.null(seed)) {
    NA_integer_
  } else {
    as.integer(seed)
  }
}

.normalize_missing_prop <- function(missing_prop) {
  if (length(missing_prop) != 1L || is.na(missing_prop)) {
    stop("`missing_prop` must be a single non-missing numeric value.", call. = FALSE)
  }

  if (missing_prop <= 0) {
    stop("`missing_prop` must be strictly positive.", call. = FALSE)
  }

  if (missing_prop > 1) {
    missing_prop <- missing_prop / 100
  }

  if (missing_prop >= 1) {
    stop("`missing_prop` must be smaller than 1 or 100%.", call. = FALSE)
  }

  as.numeric(missing_prop)
}

.as_numeric_matrix <- function(x, arg = "x") {
  if (inherits(x, "data.frame")) {
    x <- as.matrix(x)
  }

  if (!is.matrix(x)) {
    stop(sprintf("`%s` must be a matrix or data frame.", arg), call. = FALSE)
  }

  storage.mode(x) <- "double"
  x
}

PLS_lm <- function(...) {
  args <- list(...)
  if (is.null(args$naive) && !requireNamespace("plsdof", quietly = TRUE)) {
    args$naive <- TRUE
  }
  do.call(plsRglm::PLS_lm, args)
}

PLS_lm_kfoldcv <- function(...) {
  plsRglm::PLS_lm_kfoldcv(...)
}

.coerce_dataset <- function(dataset) {
  if (inherits(dataset, "misspls_dataset")) {
    return(dataset)
  }

  if (is.character(dataset) && length(dataset) == 1L) {
    if (exists(dataset, inherits = TRUE)) {
      obj <- get(dataset, inherits = TRUE)
      if (inherits(obj, "misspls_dataset")) {
        return(obj)
      }
    }

    env <- new.env(parent = emptyenv())
    utils::data(list = dataset, package = "missPLS", envir = env)
    if (!exists(dataset, envir = env, inherits = FALSE)) {
      stop(sprintf("Dataset `%s` was not found.", dataset), call. = FALSE)
    }
    obj <- get(dataset, envir = env, inherits = FALSE)
    if (!inherits(obj, "misspls_dataset")) {
      stop(sprintf("Object `%s` is not a `misspls_dataset`.", dataset), call. = FALSE)
    }
    return(obj)
  }

  if (is.list(dataset) && !is.null(dataset$x) && !is.null(dataset$y)) {
    x <- .as_numeric_matrix(dataset$x, arg = "dataset$x")
    y <- as.numeric(dataset$y)
    if (nrow(x) != length(y)) {
      stop("`dataset$x` and `dataset$y` have incompatible dimensions.", call. = FALSE)
    }
    return(structure(
      list(
        name = dataset$name %||% "custom_dataset",
        x = x,
        y = y,
        data = data.frame(y = y, x, check.names = FALSE),
        response_name = dataset$response_name %||% "y",
        predictor_names = colnames(x) %||% paste0("x", seq_len(ncol(x))),
        source = dataset$source %||% "custom input",
        preprocessing = dataset$preprocessing %||% character(0),
        notes = dataset$notes %||% character(0)
      ),
      class = "misspls_dataset"
    ))
  }

  stop("`dataset` must be a `misspls_dataset`, a dataset name, or a list with `x` and `y`.", call. = FALSE)
}

.resolve_xy <- function(x, y = NULL) {
  if (inherits(x, "misspls_dataset") || (is.character(x) && length(x) == 1L)) {
    dataset <- .coerce_dataset(x)
    return(list(
      x = dataset$x,
      y = dataset$y,
      dataset_name = dataset$name
    ))
  }

  if (is.list(x) && !is.null(x$x) && !is.null(x$y) && is.null(y)) {
    dataset <- .coerce_dataset(x)
    return(list(
      x = dataset$x,
      y = dataset$y,
      dataset_name = dataset$name
    ))
  }

  if (is.null(y)) {
    stop("`y` must be supplied when `x` is not a dataset object.", call. = FALSE)
  }

  x <- .as_numeric_matrix(x, arg = "x")
  y <- as.numeric(y)
  if (nrow(x) != length(y)) {
    stop("`x` and `y` have incompatible dimensions.", call. = FALSE)
  }

  list(x = x, y = y, dataset_name = "custom_dataset")
}

.mode_integer <- function(x) {
  x <- as.integer(stats::na.omit(x))
  if (!length(x)) {
    return(NA_integer_)
  }

  tab <- sort(table(x), decreasing = TRUE)
  tied <- as.integer(names(tab)[tab == tab[[1L]]])
  min(tied)
}

.q2_selected_ncomp <- function(q2_values, threshold = 0.0975) {
  q2_values <- as.numeric(q2_values)
  if (!length(q2_values)) {
    return(0L)
  }

  flags <- !is.na(q2_values) & q2_values > threshold
  if (!flags[1L]) {
    return(0L)
  }

  first_fail <- match(FALSE, flags, nomatch = length(flags) + 1L)
  as.integer(first_fail - 1L)
}

.criteria_column_candidates <- function(criterion) {
  switch(
    criterion,
    "AIC" = c("AIC", "AIC.naive"),
    "AIC-DoF" = c("AIC.dof", "AIC.naive", "AIC"),
    "BIC" = c("BIC", "BIC.naive"),
    "BIC-DoF" = c("BIC.dof", "BIC.naive", "BIC"),
    NULL
  )
}

.select_from_infcrit <- function(infcrit, criterion, threshold = 0.0975) {
  out <- list(
    selected_ncomp = NA_integer_,
    criterion_value = NA_real_,
    status = "ok",
    notes = ""
  )

  if (criterion %in% c("Q2-LOO", "Q2-10fold")) {
    if (!("Q2_Y" %in% colnames(infcrit))) {
      out$status <- "unavailable"
      out$notes <- sprintf("`%s` is not available in the criterion matrix.", criterion)
      return(out)
    }

    q2_values <- infcrit[-1L, "Q2_Y", drop = TRUE]
    selected_ncomp <- .q2_selected_ncomp(q2_values, threshold = threshold)
    out$selected_ncomp <- selected_ncomp
    if (selected_ncomp > 0L) {
      out$criterion_value <- unname(q2_values[selected_ncomp])
    }
    return(out)
  }

  candidates <- .criteria_column_candidates(criterion)
  if (is.null(candidates)) {
    stop(sprintf("Unsupported criterion `%s`.", criterion), call. = FALSE)
  }

  column_name <- candidates[candidates %in% colnames(infcrit)][1L]
  if (is.na(column_name) || !nzchar(column_name)) {
    out$status <- "unavailable"
    out$notes <- sprintf("`%s` is not available in the criterion matrix.", criterion)
    return(out)
  }

  values <- as.numeric(infcrit[, column_name, drop = TRUE])
  if (!any(is.finite(values))) {
    out$status <- "unavailable"
    out$notes <- sprintf("`%s` is present but contains no finite values.", column_name)
    return(out)
  }

  preferred_column <- candidates[[1L]]
  if (!identical(column_name, preferred_column)) {
    out$notes <- sprintf("`%s` was unavailable; `%s` was used instead.", preferred_column, column_name)
  }

  idx <- which.min(replace(values, !is.finite(values), Inf))
  out$selected_ncomp <- as.integer(idx - 1L)
  out$criterion_value <- values[idx]
  out
}

.make_folds <- function(n, k, seed) {
  if (k < 2L) {
    stop("`k` must be at least 2.", call. = FALSE)
  }

  k <- min(as.integer(k), as.integer(n))
  .set_seed_if_supplied(seed)
  idx <- sample.int(n)
  split(idx, rep(seq_len(k), length.out = n))
}

.complete_selection_matrix <- function(x, y, max_ncomp, criterion, seed, folds, threshold) {
  if (criterion %in% c("Q2-LOO", "Q2-10fold")) {
    k <- if (criterion == "Q2-LOO") nrow(x) else folds
    .set_seed_if_supplied(seed)
    cv_fit <- plsRglm::cv.plsR(
      object = y,
      dataX = x,
      nt = max_ncomp,
      K = k,
      verbose = FALSE
    )
    cv_infos <- plsRglm::kfolds2CVinfos_lm(cv_fit, MClassed = FALSE, verbose = FALSE)[[1L]]
    return(cv_infos)
  }

  fit <- PLS_lm(
    dataY = y,
    dataX = x,
    nt = max_ncomp,
    verbose = FALSE
  )
  fit$InfCrit
}

.incomplete_q2_cv <- function(x, y, max_ncomp, seed, folds, selection_method, threshold) {
  method_na <- if (selection_method == "nipals_standard") "missingdata" else "adaptative"
  type_vc <- if (selection_method == "nipals_standard") "standard" else "adaptative"
  fold_index <- .make_folds(nrow(x), folds, seed)
  press <- rep(0, max_ncomp)
  available <- rep(TRUE, max_ncomp)

  full_fit <- plsRglm::plsR(
    dataY = y,
    dataX = x,
    nt = max_ncomp,
    typeVC = type_vc,
    MClassed = FALSE,
    verbose = FALSE
  )
  rss_prev <- as.numeric(full_fit$InfCrit[seq_len(max_ncomp), "RSS_Y", drop = TRUE])

  for (fold in fold_index) {
    train_x <- x[-fold, , drop = FALSE]
    train_y <- y[-fold]
    test_x <- x[fold, , drop = FALSE]
    test_y <- y[fold]

    fit <- plsRglm::plsR(
      dataY = train_y,
      dataX = train_x,
      nt = max_ncomp,
      typeVC = type_vc,
      MClassed = FALSE,
      verbose = FALSE
    )

    computed_nt <- min(max_ncomp, nrow(fit$InfCrit) - 1L)
    for (comp in seq_len(max_ncomp)) {
      if (comp > computed_nt) {
        available[comp] <- FALSE
        next
      }

      pred <- plsRglm::predict.plsRmodel(
        fit,
        newdata = as.data.frame(test_x),
        comps = comp,
        methodNA = method_na,
        verbose = FALSE
      )
      press[comp] <- press[comp] + sum((test_y - as.numeric(pred[, 1L]))^2)
    }
  }

  q2 <- rep(NA_real_, max_ncomp)
  valid <- available & is.finite(rss_prev) & rss_prev > 0
  q2[valid] <- 1 - (press[valid] / rss_prev[valid])

  q2cum <- rep(NA_real_, max_ncomp)
  running_press <- 1
  running_rss <- 1
  for (comp in seq_len(max_ncomp)) {
    if (!is.finite(press[comp]) || !is.finite(rss_prev[comp]) || rss_prev[comp] <= 0) {
      next
    }
    running_press <- running_press * press[comp]
    running_rss <- running_rss * rss_prev[comp]
    q2cum[comp] <- 1 - running_press / running_rss
  }

  out <- cbind(
    Q2cum_Y = c(NA_real_, q2cum),
    LimQ2_Y = c(NA_real_, rep(threshold, max_ncomp)),
    Q2_Y = c(NA_real_, q2),
    PRESS_Y = c(NA_real_, press),
    RSS_Y = full_fit$InfCrit[, "RSS_Y", drop = TRUE]
  )
  rownames(out) <- rownames(full_fit$InfCrit)
  out
}

.selection_result <- function(selection_method, criterion, selected_ncomp, criterion_value,
                              max_ncomp, seed, status = "ok", notes = "", n_imputations = 1L) {
  data.frame(
    selection_method = selection_method,
    criterion = criterion,
    selected_ncomp = as.integer(selected_ncomp),
    criterion_value = as.numeric(criterion_value),
    max_ncomp = as.integer(max_ncomp),
    seed = .seed_column_value(seed),
    n_imputations = as.integer(n_imputations),
    status = status,
    notes = notes,
    stringsAsFactors = FALSE
  )
}

.study_row <- function(study, dataset, n, p, true_ncomp, target_ncomp, mechanism,
                       missing_prop, method, criterion, selected_ncomp,
                       matched_target, runtime_sec, seed, status, notes) {
  data.frame(
    study = study,
    dataset = dataset,
    n = as.integer(n),
    p = as.integer(p),
    true_ncomp = if (is.na(true_ncomp)) NA_integer_ else as.integer(true_ncomp),
    target_ncomp = if (is.na(target_ncomp)) NA_integer_ else as.integer(target_ncomp),
    mechanism = if (is.null(mechanism)) NA_character_ else as.character(mechanism),
    missing_prop = if (is.na(missing_prop)) NA_real_ else as.numeric(missing_prop),
    method = method,
    criterion = criterion,
    selected_ncomp = if (is.na(selected_ncomp)) NA_integer_ else as.integer(selected_ncomp),
    matched_target = if (is.na(matched_target)) NA else as.logical(matched_target),
    runtime_sec = as.numeric(runtime_sec),
    seed = .seed_column_value(seed),
    status = status,
    notes = notes,
    stringsAsFactors = FALSE
  )
}

.imputation_method_label <- function(method) {
  switch(
    method,
    mice = "MICE",
    knn = "KNNimpute",
    svd = "SVDimpute",
    method
  )
}

.selection_method_label <- function(method) {
  switch(
    method,
    complete = "Complete",
    nipals_standard = "NIPALS-PLSR (standard)",
    nipals_adaptative = "NIPALS-PLSR (adaptative)",
    nipals_adaptive = "NIPALS-PLSR (adaptative)",
    method
  )
}

.normalise_selection_method <- function(method) {
  method <- match.arg(
    method,
    choices = c("complete", "nipals_standard", "nipals_adaptative", "nipals_adaptive")
  )
  if (method == "nipals_adaptive") {
    return("nipals_adaptative")
  }
  method
}

.vif_values <- function(x) {
  x <- .as_numeric_matrix(x)
  if (nrow(x) <= ncol(x)) {
    return(rep(NA_real_, ncol(x)))
  }

  out <- rep(NA_real_, ncol(x))
  for (j in seq_len(ncol(x))) {
    response <- x[, j]
    predictors <- x[, -j, drop = FALSE]
    fit <- tryCatch(
      stats::lm.fit(cbind(1, predictors), response),
      error = function(e) NULL
    )
    if (is.null(fit)) {
      next
    }
    rss <- sum(fit$residuals^2)
    tss <- sum((response - mean(response))^2)
    r_squared <- 1 - (rss / tss)
    if (is.finite(r_squared) && r_squared < 1) {
      out[j] <- 1 / (1 - r_squared)
    } else {
      out[j] <- Inf
    }
  }
  out
}

test_that("complete-data selection matches direct plsRglm outputs", {
  sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
  PLS_lm <- missPLS:::PLS_lm

  sel_aic <- select_ncomp(sim$x, sim$y, method = "complete", criterion = "AIC", max_ncomp = 4, seed = 2)
  fit <- PLS_lm(dataY = sim$y, dataX = sim$x, nt = 4, verbose = FALSE)
  direct_aic <- unname(which.min(fit$InfCrit[, "AIC"]) - 1L)
  expect_identical(sel_aic$selected_ncomp, direct_aic)

  sel_q2 <- select_ncomp(sim$x, sim$y, method = "complete", criterion = "Q2-10fold", max_ncomp = 4, seed = 2, folds = 5)
  PLS_lm_kfoldcv <- missPLS:::PLS_lm_kfoldcv
  set.seed(2)
  cv <- plsRglm::kfolds2CVinfos_lm(
    plsRglm::cv.plsR(object = sim$y, dataX = sim$x, nt = 4, K = 5, verbose = FALSE),
    MClassed = FALSE,
    verbose = FALSE
  )[[1]]
  direct_q2 <- missPLS:::.q2_selected_ncomp(cv[-1, "Q2_Y"], threshold = 0.0975)
  expect_identical(sel_q2$selected_ncomp, direct_q2)
})

test_that("complete-data Q2 uses the documented cv.plsR interface", {
  sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
  PLS_lm <- missPLS:::PLS_lm
  PLS_lm_kfoldcv <- missPLS:::PLS_lm_kfoldcv

  set.seed(11)
  via_cv_plsR <- plsRglm::kfolds2CVinfos_lm(
    plsRglm::cv.plsR(object = sim$y, dataX = sim$x, nt = 4, K = 5, verbose = FALSE),
    MClassed = FALSE,
    verbose = FALSE
  )[[1]]

  set.seed(11)
  via_kfold <- plsRglm::kfolds2CVinfos_lm(
    plsRglm::PLS_lm_kfoldcv(dataY = sim$y, dataX = sim$x, nt = 4, K = 5, verbose = FALSE),
    MClassed = FALSE,
    verbose = FALSE
  )[[1]]

  expect_identical(via_cv_plsR, via_kfold)
})

test_that("incomplete-data selection returns a stable schema", {
  sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
  miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
  sel <- select_ncomp(
    x = miss$x_incomplete,
    y = sim$y,
    method = "nipals_standard",
    criterion = "Q2-LOO",
    max_ncomp = 4,
    seed = 4
  )

  expect_named(sel, c("selection_method", "criterion", "selected_ncomp", "criterion_value",
    "max_ncomp", "seed", "n_imputations", "status", "notes"))
  expect_equal(nrow(sel), 1L)
})

#' Simulate PLS data
#'
#' Simulate a univariate-response PLS dataset using the Li et al.-style
#' generator available in `plsRglm`.
#'
#' @param n Number of observations.
#' @param p Number of predictors.
#' @param true_ncomp True number of latent components.
#' @param seed Optional random seed. If supplied, it is used only for this call.
#' @param model Simulation model. Only `"li2002"` is currently supported.
#'
#' @return A list with components `x`, `y`, `data`, `true_ncomp`, `seed`, and
#'   `model`.
#' @export
#'
#' @examples
#' sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 42)
#' str(sim)
simulate_pls_data <- function(n, p, true_ncomp, seed = NULL, model = "li2002") {
  if (!identical(model, "li2002")) {
    stop("Only `model = \"li2002\"` is currently supported.", call. = FALSE)
  }

  n <- as.integer(n)
  p <- as.integer(p)
  true_ncomp <- as.integer(true_ncomp)
  seed <- .set_seed_if_supplied(seed)

  if (n < 2L) {
    stop("`n` must be at least 2.", call. = FALSE)
  }
  if (p < 2L) {
    stop("`p` must be at least 2.", call. = FALSE)
  }
  if (true_ncomp < 2L || true_ncomp > min(6L, p)) {
    stop("`true_ncomp` must be between 2 and `min(6, p)`.", call. = FALSE)
  }

  sim <- t(replicate(n, plsRglm::simul_data_UniYX(p, true_ncomp)))
  colnames(sim) <- c("y", paste0("x", seq_len(p)))

  x <- sim[, -1L, drop = FALSE]
  y <- as.numeric(sim[, 1L])

  list(
    x = x,
    y = y,
    data = data.frame(y = y, x, check.names = FALSE),
    true_ncomp = true_ncomp,
    seed = .seed_column_value(seed),
    model = model
  )
}

#' Run a real-data study
#'
#' @param dataset A packaged dataset name or `misspls_dataset` object.
#' @param seed Optional base random seed.
#' @param missing_props Missing-data proportions as fractions or percentages.
#' @param mechanisms Missing-data mechanisms.
#' @param reps Number of replicate missingness draws for each mechanism and
#'   proportion.
#' @param baseline_reps Number of repeated complete-data `Q2-10fold` selections
#'   used to determine `t**`.
#' @param max_ncomp Maximum number of extracted components.
#' @param criteria Criteria evaluated on incomplete and imputed data.
#' @param incomplete_methods Incomplete-data NIPALS workflows.
#' @param imputation_methods Imputation methods.
#' @param folds Number of folds used by `"Q2-10fold"`.
#' @param mar_y_bias MAR bias parameter passed to [add_missingness()].
#'
#' @return A data frame with one row per study run.
#' @export
run_real_data_study <- function(
    dataset,
    seed = NULL,
    missing_props = seq(5, 50, 5),
    mechanisms = c("MCAR", "MAR"),
    reps = 1L,
    baseline_reps = 100L,
    max_ncomp = 12L,
    criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
    incomplete_methods = c("nipals_standard", "nipals_adaptative"),
    imputation_methods = c("mice", "knn", "svd"),
    folds = 10L,
    mar_y_bias = 0.8) {
  dataset <- .coerce_dataset(dataset)
  seed <- .normalize_seed(seed, allow_null = TRUE)
  reps <- as.integer(reps)
  baseline_reps <- as.integer(baseline_reps)

  rows <- vector("list", 0L)
  row_id <- 0L

  baseline_selections <- vector("list", baseline_reps)
  for (i in seq_len(baseline_reps)) {
    selection_seed <- .seed_with_offset(seed, i)
    timing <- system.time(
      baseline_selections[[i]] <- select_ncomp(
        x = dataset$x,
        y = dataset$y,
        method = "complete",
        criterion = "Q2-10fold",
        max_ncomp = max_ncomp,
        seed = selection_seed,
        folds = folds
      )
    )
    row_id <- row_id + 1L
    rows[[row_id]] <- .study_row(
      study = "real_data",
      dataset = dataset$name,
      n = nrow(dataset$x),
      p = ncol(dataset$x),
      true_ncomp = NA_integer_,
      target_ncomp = baseline_selections[[i]]$selected_ncomp,
      mechanism = NA_character_,
      missing_prop = 0,
      method = "Complete",
      criterion = "Q2-10fold",
      selected_ncomp = baseline_selections[[i]]$selected_ncomp,
      matched_target = NA,
      runtime_sec = unname(timing["elapsed"]),
      seed = selection_seed,
      status = baseline_selections[[i]]$status,
      notes = baseline_selections[[i]]$notes
    )
  }

  target_ncomp <- .mode_integer(vapply(baseline_selections, function(x) x$selected_ncomp, integer(1L)))

  for (mechanism in mechanisms) {
    for (missing_prop in missing_props) {
      for (rep_id in seq_len(reps)) {
        miss_seed <- .seed_with_offset(seed, 1000L * rep_id + match(mechanism, mechanisms) * 100L)
        miss <- add_missingness(
          x = dataset$x,
          y = dataset$y,
          mechanism = mechanism,
          missing_prop = missing_prop,
          seed = miss_seed,
          mar_y_bias = mar_y_bias
        )

        for (selection_method in incomplete_methods) {
          for (criterion in criteria) {
            selection_seed <- .seed_with_offset(miss_seed, row_id + 11L)
            timing <- system.time(
              sel <- select_ncomp(
                x = miss$x_incomplete,
                y = dataset$y,
                method = selection_method,
                criterion = criterion,
                max_ncomp = max_ncomp,
                seed = selection_seed,
                folds = folds
              )
            )
            row_id <- row_id + 1L
            rows[[row_id]] <- .study_row(
              study = "real_data",
              dataset = dataset$name,
              n = nrow(dataset$x),
              p = ncol(dataset$x),
              true_ncomp = NA_integer_,
              target_ncomp = target_ncomp,
              mechanism = mechanism,
              missing_prop = .normalize_missing_prop(missing_prop) * 100,
              method = .selection_method_label(selection_method),
              criterion = criterion,
              selected_ncomp = sel$selected_ncomp,
              matched_target = !is.na(sel$selected_ncomp) && sel$selected_ncomp == target_ncomp,
              runtime_sec = unname(timing["elapsed"]),
              seed = selection_seed,
              status = sel$status,
              notes = sel$notes
            )
          }
        }

        for (imp_method in imputation_methods) {
          imputation_seed <- .seed_with_offset(miss_seed, 50000L + row_id)
          imp <- tryCatch(
            impute_pls_data(
              x = miss$x_incomplete,
              method = imp_method,
              seed = imputation_seed,
              m = max(1L, as.integer(round(.normalize_missing_prop(missing_prop) * 100)))
            ),
            error = function(e) e
          )

          for (criterion in criteria) {
            if (inherits(imp, "error")) {
              sel <- .selection_result(
                selection_method = paste0("complete_", imp_method),
                criterion = criterion,
                selected_ncomp = NA_integer_,
                criterion_value = NA_real_,
                max_ncomp = max_ncomp,
                seed = imputation_seed,
                status = "error",
                notes = conditionMessage(imp)
              )
              elapsed <- NA_real_
              selection_seed <- imputation_seed
            } else {
              selection_seed <- .seed_with_offset(imputation_seed, match(criterion, criteria))
              timing <- system.time(
                sel <- select_ncomp(
                  x = imp,
                  y = dataset$y,
                  method = "complete",
                  criterion = criterion,
                  max_ncomp = max_ncomp,
                  seed = selection_seed,
                  folds = folds
                )
              )
              elapsed <- unname(timing["elapsed"])
            }

            row_id <- row_id + 1L
            rows[[row_id]] <- .study_row(
              study = "real_data",
              dataset = dataset$name,
              n = nrow(dataset$x),
              p = ncol(dataset$x),
              true_ncomp = NA_integer_,
              target_ncomp = target_ncomp,
              mechanism = mechanism,
              missing_prop = .normalize_missing_prop(missing_prop) * 100,
              method = .imputation_method_label(imp_method),
              criterion = criterion,
              selected_ncomp = sel$selected_ncomp,
              matched_target = !is.na(sel$selected_ncomp) && sel$selected_ncomp == target_ncomp,
              runtime_sec = elapsed,
              seed = selection_seed,
              status = sel$status,
              notes = sel$notes
            )
          }
        }
      }
    }
  }

  do.call(rbind, rows)
}

test_that("simulation study returns the expected schema and summarises cleanly", {
  results <- run_simulation_study(
    dimensions = list(c(25, 10)),
    true_ncomp = 2,
    missing_props = 10,
    mechanisms = "MCAR",
    reps = 3,
    seed = 1,
    max_ncomp = 4,
    criteria = c("Q2-LOO", "AIC"),
    incomplete_methods = "nipals_standard",
    imputation_methods = "knn",
    folds = 5
  )

  expect_named(results, c(
    "study", "dataset", "n", "p", "true_ncomp", "target_ncomp", "mechanism",
    "missing_prop", "method", "criterion", "selected_ncomp", "matched_target",
    "runtime_sec", "seed", "status", "notes"
  ))

  summary_df <- summarize_simulation_study(results)
  expect_true(nrow(summary_df) > 0)

  complete_rows <- results[results$method == "Complete" & results$status == "ok", , drop = FALSE]
  q2_rate <- mean(complete_rows$matched_target[complete_rows$criterion == "Q2-LOO"])
  aic_rate <- mean(complete_rows$matched_target[complete_rows$criterion == "AIC"])
  expect_true(is.finite(q2_rate))
  expect_true(is.finite(aic_rate))
  expect_true(q2_rate >= aic_rate)
})

test_that("real-data study smoke test runs on bromhexine", {
  results <- run_real_data_study(
    dataset = "bromhexine",
    seed = 1,
    missing_props = 5,
    mechanisms = "MCAR",
    reps = 1,
    baseline_reps = 3,
    max_ncomp = 4,
    criteria = c("Q2-10fold"),
    incomplete_methods = "nipals_standard",
    imputation_methods = "knn",
    folds = 5
  )

  expect_true(nrow(results) > 0)
  expect_true(any(results$dataset == "bromhexine"))
})

test_that("study runners do not impose a default seed", {
  set.seed(303)
  results <- run_simulation_study(
    dimensions = list(c(20, 10)),
    true_ncomp = 2,
    missing_props = numeric(0),
    mechanisms = character(0),
    reps = 1,
    seed = NULL,
    max_ncomp = 3,
    criteria = "AIC"
  )

  expect_true(nrow(results) > 0)
  expect_true(all(is.na(results$seed)))
})

#' Run a simulation study
#'
#' Run the simulation workflows used in the article and thesis.
#'
#' @param dimensions List of `(n, p)` integer pairs.
#' @param true_ncomp Vector of true component counts.
#' @param missing_props Missing-data proportions as fractions or percentages.
#' @param mechanisms Missing-data mechanisms.
#' @param reps Number of replicates.
#' @param seed Optional base random seed.
#' @param max_ncomp Maximum number of extracted components.
#' @param criteria Criteria evaluated on complete and imputed data.
#' @param incomplete_methods Incomplete-data NIPALS workflows.
#' @param imputation_methods Imputation methods.
#' @param folds Number of folds used by `"Q2-10fold"`.
#' @param mar_y_bias MAR bias parameter passed to [add_missingness()].
#'
#' @return A data frame with one row per study run.
#' @export
run_simulation_study <- function(
    dimensions = list(c(500L, 100L), c(500L, 20L), c(100L, 20L), c(80L, 25L),
                      c(60L, 33L), c(40L, 50L), c(20L, 100L)),
    true_ncomp = c(2L, 4L, 6L),
    missing_props = seq(5, 50, 5),
    mechanisms = c("MCAR", "MAR"),
    reps = 1L,
    seed = NULL,
    max_ncomp = 8L,
    criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
    incomplete_methods = c("nipals_standard", "nipals_adaptative"),
    imputation_methods = c("mice", "knn", "svd"),
    folds = 10L,
    mar_y_bias = 0.8) {
  seed <- .normalize_seed(seed, allow_null = TRUE)
  reps <- as.integer(reps)
  max_ncomp <- as.integer(max_ncomp)

  rows <- vector("list", 0L)
  row_id <- 0L
  counter <- 0L

  for (dim_pair in dimensions) {
    n <- as.integer(dim_pair[[1L]])
    p <- as.integer(dim_pair[[2L]])
    dataset_name <- sprintf("sim_n%d_p%d", n, p)

    for (target in as.integer(true_ncomp)) {
      for (rep_id in seq_len(reps)) {
        counter <- counter + 1L
        sim_seed <- .seed_with_offset(seed, counter * 1000L)
        sim <- simulate_pls_data(n = n, p = p, true_ncomp = target, seed = sim_seed)

        for (criterion in criteria) {
          selection_seed <- .seed_with_offset(sim_seed, match(criterion, criteria))
          timing <- system.time(
            sel <- select_ncomp(
              x = sim$x,
              y = sim$y,
              method = "complete",
              criterion = criterion,
              max_ncomp = max_ncomp,
              seed = selection_seed,
              folds = folds
            )
          )
          row_id <- row_id + 1L
          rows[[row_id]] <- .study_row(
            study = "simulation",
            dataset = dataset_name,
            n = n,
            p = p,
            true_ncomp = target,
            target_ncomp = target,
            mechanism = NA_character_,
            missing_prop = 0,
            method = "Complete",
            criterion = criterion,
            selected_ncomp = sel$selected_ncomp,
            matched_target = !is.na(sel$selected_ncomp) && sel$selected_ncomp == target,
            runtime_sec = unname(timing["elapsed"]),
            seed = selection_seed,
            status = sel$status,
            notes = sel$notes
          )
        }

        if (!length(missing_props) || !length(mechanisms)) {
          next
        }

        for (mechanism in mechanisms) {
          for (missing_prop in missing_props) {
            miss_seed <- .seed_with_offset(sim_seed, as.integer(round(.normalize_missing_prop(missing_prop) * 1000)) +
              if (mechanism == "MAR") 50000L else 0L)
            miss <- add_missingness(
              x = sim$x,
              y = sim$y,
              mechanism = mechanism,
              missing_prop = missing_prop,
              seed = miss_seed,
              mar_y_bias = mar_y_bias
            )

            for (selection_method in incomplete_methods) {
              for (criterion in criteria) {
                selection_seed <- .seed_with_offset(miss_seed, row_id + 17L)
                timing <- system.time(
                  sel <- select_ncomp(
                    x = miss$x_incomplete,
                    y = sim$y,
                    method = selection_method,
                    criterion = criterion,
                    max_ncomp = max_ncomp,
                    seed = selection_seed,
                    folds = folds
                  )
                )
                row_id <- row_id + 1L
                rows[[row_id]] <- .study_row(
                  study = "simulation",
                  dataset = dataset_name,
                  n = n,
                  p = p,
                  true_ncomp = target,
                  target_ncomp = target,
                  mechanism = mechanism,
                  missing_prop = .normalize_missing_prop(missing_prop) * 100,
                  method = .selection_method_label(selection_method),
                  criterion = criterion,
                  selected_ncomp = sel$selected_ncomp,
                  matched_target = !is.na(sel$selected_ncomp) && sel$selected_ncomp == target,
                  runtime_sec = unname(timing["elapsed"]),
                  seed = selection_seed,
                  status = sel$status,
                  notes = sel$notes
                )
              }
            }

            for (imp_method in imputation_methods) {
              imputation_seed <- .seed_with_offset(miss_seed, 100000L + row_id)
              imp <- tryCatch(
                impute_pls_data(
                  x = miss$x_incomplete,
                  method = imp_method,
                  seed = imputation_seed,
                  m = max(1L, as.integer(round(.normalize_missing_prop(missing_prop) * 100)))
                ),
                error = function(e) e
              )

              for (criterion in criteria) {
                if (inherits(imp, "error")) {
                  sel <- .selection_result(
                    selection_method = paste0("complete_", imp_method),
                    criterion = criterion,
                    selected_ncomp = NA_integer_,
                    criterion_value = NA_real_,
                    max_ncomp = max_ncomp,
                    seed = imputation_seed,
                    status = "error",
                    notes = conditionMessage(imp)
                  )
                  elapsed <- NA_real_
                } else {
                  selection_seed <- .seed_with_offset(imputation_seed, match(criterion, criteria))
                  timing <- system.time(
                    sel <- select_ncomp(
                      x = imp,
                      y = sim$y,
                      method = "complete",
                      criterion = criterion,
                      max_ncomp = max_ncomp,
                      seed = selection_seed,
                      folds = folds
                    )
                  )
                  elapsed <- unname(timing["elapsed"])
                }

                row_id <- row_id + 1L
                rows[[row_id]] <- .study_row(
                  study = "simulation",
                  dataset = dataset_name,
                  n = n,
                  p = p,
                  true_ncomp = target,
                  target_ncomp = target,
                  mechanism = mechanism,
                  missing_prop = .normalize_missing_prop(missing_prop) * 100,
                  method = .imputation_method_label(imp_method),
                  criterion = criterion,
                  selected_ncomp = sel$selected_ncomp,
                  matched_target = !is.na(sel$selected_ncomp) && sel$selected_ncomp == target,
                  runtime_sec = elapsed,
                  seed = if (inherits(imp, "error")) imputation_seed else selection_seed,
                  status = sel$status,
                  notes = sel$notes
                )
              }
            }
          }
        }
      }
    }
  }

  do.call(rbind, rows)
}

#' Summarize simulation or real-data study results
#'
#' @param results A results data frame returned by [run_simulation_study()] or
#'   [run_real_data_study()].
#'
#' @return A grouped summary data frame.
#' @export
#'
#' @examples
#' sim_results <- run_simulation_study(
#'   dimensions = list(c(30, 12)),
#'   true_ncomp = 2,
#'   missing_props = numeric(0),
#'   mechanisms = character(0),
#'   reps = 2,
#'   seed = 1
#' )
#' summarize_simulation_study(sim_results)
summarize_simulation_study <- function(results) {
  required <- c(
    "study", "dataset", "n", "p", "true_ncomp", "target_ncomp", "mechanism",
    "missing_prop", "method", "criterion", "selected_ncomp", "matched_target",
    "runtime_sec", "seed", "status", "notes"
  )
  missing_cols <- setdiff(required, colnames(results))
  if (length(missing_cols)) {
    stop(sprintf("`results` is missing required columns: %s", paste(missing_cols, collapse = ", ")),
      call. = FALSE
    )
  }

  group_cols <- c("study", "dataset", "n", "p", "true_ncomp", "target_ncomp",
                  "mechanism", "missing_prop", "method", "criterion")

  group_frame <- results[, group_cols, drop = FALSE]
  for (nm in names(group_frame)) {
    is_missing <- is.na(group_frame[[nm]])
    group_frame[[nm]] <- as.character(group_frame[[nm]])
    group_frame[[nm]][is_missing] <- "<NA>"
  }

  split_ids <- split(seq_len(nrow(results)), do.call(interaction, c(group_frame, drop = TRUE)))
  rows <- lapply(split_ids, function(idx) {
    chunk <- results[idx, , drop = FALSE]
    ok <- chunk$status == "ok"
    first <- chunk[1L, group_cols, drop = FALSE]
    data.frame(
      first,
      n_runs = nrow(chunk),
      n_success = sum(ok, na.rm = TRUE),
      n_matched = sum(chunk$matched_target %in% TRUE, na.rm = TRUE),
      match_rate = if (sum(ok, na.rm = TRUE) > 0) {
        sum(chunk$matched_target %in% TRUE, na.rm = TRUE) / sum(ok, na.rm = TRUE)
      } else {
        NA_real_
      },
      mean_selected_ncomp = if (any(ok)) mean(chunk$selected_ncomp[ok], na.rm = TRUE) else NA_real_,
      median_selected_ncomp = if (any(ok)) stats::median(chunk$selected_ncomp[ok], na.rm = TRUE) else NA_real_,
      mean_runtime_sec = mean(chunk$runtime_sec, na.rm = TRUE),
      status_summary = paste(sort(unique(chunk$status)), collapse = ","),
      stringsAsFactors = FALSE
    )
  })

  summary_df <- do.call(rbind, rows)
  summary_df$mechanism[summary_df$mechanism == "<NA>"] <- NA_character_
  summary_df$missing_prop[summary_df$missing_prop == "<NA>"] <- NA_character_
  summary_df$true_ncomp[summary_df$true_ncomp == "<NA>"] <- NA_character_
  summary_df$target_ncomp[summary_df$target_ncomp == "<NA>"] <- NA_character_
  summary_df
}

#!/usr/bin/env Rscript

source("tools/repro_helpers.R")
load_misspls()

baseline_reps <- as.integer(Sys.getenv("MISSPLS_BASELINE_REPS", "100"))
output_root <- Sys.getenv("MISSPLS_OUTPUT_DIR", unset = "")
if (!nzchar(output_root)) {
  output_root <- tempdir()
}
output_dir <- make_output_dir("real-data", root_dir = output_root)

datasets <- c("bromhexine", "tetracycline", "ozone_complete", "octane")
results <- do.call(rbind, lapply(seq_along(datasets), function(i) {
  missPLS::run_real_data_study(
    dataset = datasets[[i]],
    seed = 20250407L + i,
    missing_props = seq(5, 50, 5),
    mechanisms = c("MCAR", "MAR"),
    reps = 1L,
    baseline_reps = baseline_reps,
    max_ncomp = 12L,
    criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
    incomplete_methods = c("nipals_standard", "nipals_adaptative"),
    imputation_methods = c("mice", "knn", "svd"),
    folds = 10L
  )
}))

summary_df <- missPLS::summarize_simulation_study(results)
save_outputs(output_dir, "real-data", results, summary_df)
write_manifest(output_dir, list(
  baseline_reps = baseline_reps,
  script = "run_real_data_studies.R",
  datasets = datasets
))
write_match_plot(output_dir, "real-data", summary_df, "Real-data study match rates")
write_discrepancy_report(output_dir, "Real data studies", summary_df)

#' Impute a predictor matrix
#'
#' Apply one of the imputation strategies used in the article and thesis.
#'
#' @param x Incomplete predictor matrix or data frame.
#' @param method Imputation method: `"mice"`, `"knn"`, or `"svd"`.
#' @param seed Optional random seed forwarded to stochastic imputers when
#'   supported.
#' @param m Number of imputations for `method = "mice"`. By default this is set
#'   to the missing-data percentage rounded to the nearest integer.
#' @param k Number of neighbours for `method = "knn"`.
#' @param svd_rank Target rank for `method = "svd"`.
#' @param svd_maxiter Maximum number of iterations for the fallback SVD routine.
#'
#' @return A `misspls_imputation` object.
#' @export
#'
#' @examples
#' sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 1)
#' miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
#' imp <- impute_pls_data(miss$x_incomplete, method = "knn", seed = 3)
#' length(imp$datasets)
impute_pls_data <- function(x, method = c("mice", "knn", "svd"), seed = NULL,
                            m, k = 15L, svd_rank = 10L, svd_maxiter = 1000L) {
  method <- match.arg(method)
  seed <- .normalize_seed(seed, allow_null = TRUE)
  x <- .as_numeric_matrix(x)
  predictor_names <- colnames(x) %||% paste0("x", seq_len(ncol(x)))
  colnames(x) <- predictor_names

  if (missing(m)) {
    m <- max(1L, as.integer(round(mean(is.na(x)) * 100)))
  }
  m <- as.integer(m)
  k <- as.integer(k)
  svd_rank <- as.integer(svd_rank)
  svd_maxiter <- as.integer(svd_maxiter)

  impute_svd_fallback <- function(x_in, rank_k, maxiter) {
    obs_mask <- !is.na(x_in)
    x_imp <- x_in
    col_means <- colMeans(x_imp, na.rm = TRUE)
    for (j in seq_len(ncol(x_imp))) {
      x_imp[!obs_mask[, j], j] <- col_means[j]
    }

    rank_k <- min(rank_k, nrow(x_imp), ncol(x_imp))
    for (iter in seq_len(maxiter)) {
      old_missing <- x_imp[!obs_mask]
      sv <- svd(x_imp)
      reconstructed <- sv$u[, seq_len(rank_k), drop = FALSE] %*%
        diag(sv$d[seq_len(rank_k)], nrow = rank_k) %*%
        t(sv$v[, seq_len(rank_k), drop = FALSE])
      x_imp[!obs_mask] <- reconstructed[!obs_mask]
      if (max(abs(x_imp[!obs_mask] - old_missing), na.rm = TRUE) < 1e-8) {
        break
      }
    }

    x_imp
  }

  datasets <- switch(
    method,
    mice = {
      mids <- suppressWarnings(mice::mice(
        data = as.data.frame(x),
        m = m,
        method = "norm",
        printFlag = FALSE,
        seed = if (is.null(seed)) NA_integer_ else seed,
        remove_collinear = FALSE
      ))
      lapply(seq_len(m), function(i) {
        out <- as.matrix(mice::complete(mids, action = i))
        colnames(out) <- predictor_names
        storage.mode(out) <- "double"
        out
      })
    },
    knn = {
      out <- as.matrix(VIM::kNN(as.data.frame(x), k = k, imp_var = FALSE))
      colnames(out) <- predictor_names
      storage.mode(out) <- "double"
      list(out)
    },
    svd = {
      out <- if (requireNamespace("bcv", quietly = TRUE)) {
        suppressWarnings(bcv::impute.svd(x, k = svd_rank, maxiter = svd_maxiter)$x)
      } else {
        impute_svd_fallback(x, rank_k = svd_rank, maxiter = svd_maxiter)
      }
      out <- as.matrix(out)
      colnames(out) <- predictor_names
      storage.mode(out) <- "double"
      list(out)
    }
  )

  structure(
    list(
      method = method,
      seed = .seed_column_value(seed),
      datasets = datasets,
      n_imputations = length(datasets),
      predictor_names = predictor_names
    ),
    class = "misspls_imputation"
  )
}

#' Select the number of PLS components
#'
#' Select the number of components for complete, imputed, or incomplete-data
#' PLS workflows.
#'
#' @param x Predictor matrix, dataset object, or `misspls_imputation` object.
#' @param y Numeric response vector. This may be omitted when `x` already
#'   contains a response.
#' @param method Selection workflow: `"complete"`, `"nipals_standard"`, or
#'   `"nipals_adaptative"`.
#' @param criterion Selection criterion: `"Q2-LOO"`, `"Q2-10fold"`, `"AIC"`,
#'   `"AIC-DoF"`, `"BIC"`, or `"BIC-DoF"`.
#' @param max_ncomp Maximum number of components to consider.
#' @param seed Optional random seed used by the cross-validation and imputation
#'   aggregation steps.
#' @param folds Number of cross-validation folds used by `"Q2-10fold"`.
#' @param threshold Threshold applied to `Q2` criteria.
#'
#' @return A one-row data frame describing the selected component count.
#' @export
#'
#' @examples
#' sim <- simulate_pls_data(n = 25, p = 10, true_ncomp = 2, seed = 1)
#' select_ncomp(sim$x, sim$y, method = "complete", criterion = "AIC", max_ncomp = 4, seed = 2)
select_ncomp <- function(x, y, method = c("complete", "nipals_standard", "nipals_adaptative"),
                         criterion = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
                         max_ncomp, seed = NULL, folds = 10L, threshold = 0.0975) {
  criterion <- match.arg(criterion)
  method <- .normalise_selection_method(method)
  seed <- .normalize_seed(seed, allow_null = TRUE)
  folds <- as.integer(folds)

  if (missing(max_ncomp)) {
    resolved <- if (inherits(x, "misspls_imputation")) {
      .resolve_xy(x$datasets[[1L]], y)
    } else {
      .resolve_xy(x, y)
    }
    max_ncomp <- min(8L, nrow(resolved$x) - 1L, ncol(resolved$x))
  }
  max_ncomp <- as.integer(max_ncomp)

  if (inherits(x, "misspls_imputation")) {
    if (!identical(method, "complete")) {
      return(.selection_result(
        selection_method = method,
        criterion = criterion,
        selected_ncomp = NA_integer_,
        criterion_value = NA_real_,
        max_ncomp = max_ncomp,
        seed = seed,
        status = "unavailable",
        notes = "Imputation objects can only be evaluated with `method = \"complete\"`.",
        n_imputations = x$n_imputations
      ))
    }

    selections <- lapply(seq_along(x$datasets), function(i) {
      select_ncomp(
        x = x$datasets[[i]],
        y = y,
        method = "complete",
        criterion = criterion,
        max_ncomp = max_ncomp,
        seed = .seed_with_offset(seed, i),
        folds = folds,
        threshold = threshold
      )
    })
    selections_df <- do.call(rbind, selections)
    ok <- selections_df$status == "ok"
    if (!any(ok)) {
      return(.selection_result(
        selection_method = method,
        criterion = criterion,
        selected_ncomp = NA_integer_,
        criterion_value = NA_real_,
        max_ncomp = max_ncomp,
        seed = seed,
        status = "error",
        notes = paste(unique(selections_df$notes), collapse = "; "),
        n_imputations = x$n_imputations
      ))
    }

    selected_ncomp <- .mode_integer(selections_df$selected_ncomp[ok])
    criterion_value <- mean(selections_df$criterion_value[ok], na.rm = TRUE)
    return(.selection_result(
      selection_method = paste0("complete_", x$method),
      criterion = criterion,
      selected_ncomp = selected_ncomp,
      criterion_value = criterion_value,
      max_ncomp = max_ncomp,
      seed = seed,
      status = "ok",
      notes = sprintf("Mode across %d imputed datasets.", x$n_imputations),
      n_imputations = x$n_imputations
    ))
  }

  resolved <- .resolve_xy(x, y)
  x <- resolved$x
  y <- resolved$y

  if (max_ncomp < 1L) {
    stop("`max_ncomp` must be at least 1.", call. = FALSE)
  }
  max_ncomp <- min(max_ncomp, nrow(x) - 1L, ncol(x))

  result <- tryCatch(
    {
      if (method == "complete") {
        if (anyNA(x)) {
          stop("`x` contains missing values; use an imputation workflow or NIPALS selection.")
        }
        infcrit <- .complete_selection_matrix(
          x = x,
          y = y,
          max_ncomp = max_ncomp,
          criterion = criterion,
          seed = seed,
          folds = folds,
          threshold = threshold
        )
      } else {
        fit <- plsRglm::plsR(
          dataY = y,
          dataX = x,
          nt = max_ncomp,
          typeVC = if (method == "nipals_standard") "standard" else "adaptative",
          MClassed = FALSE,
          verbose = FALSE
        )

        infcrit <- if (criterion == "Q2-10fold") {
          .incomplete_q2_cv(
            x = x,
            y = y,
            max_ncomp = max_ncomp,
            seed = seed,
            folds = folds,
            selection_method = method,
            threshold = threshold
          )
        } else {
          fit$InfCrit
        }
      }

      sel <- .select_from_infcrit(infcrit, criterion = criterion, threshold = threshold)
      .selection_result(
        selection_method = method,
        criterion = criterion,
        selected_ncomp = sel$selected_ncomp,
        criterion_value = sel$criterion_value,
        max_ncomp = max_ncomp,
        seed = seed,
        status = sel$status,
        notes = sel$notes
      )
    },
    error = function(e) {
      .selection_result(
        selection_method = method,
        criterion = criterion,
        selected_ncomp = NA_integer_,
        criterion_value = NA_real_,
        max_ncomp = max_ncomp,
        seed = seed,
        status = "error",
        notes = conditionMessage(e)
      )
    }
  )

  result
}

#' @export
print.misspls_imputation <- function(x, ...) {
  cat("<misspls_imputation>\n")
  cat("  method:", x$method, "\n")
  cat("  n_imputations:", x$n_imputations, "\n")
  cat("  predictors:", length(x$predictor_names), "\n")
  invisible(x)
}

#' @export
print.misspls_dataset <- function(x, ...) {
  cat("<misspls_dataset>\n")
  cat("  name:", x$name, "\n")
  cat("  dimensions:", nrow(x$x), "x", ncol(x$x), "\n")
  cat("  source:", x$source, "\n")
  invisible(x)
}

test_that("packaged datasets have the expected dimensions", {
  data("bromhexine", package = "missPLS", envir = environment())
  data("tetracycline", package = "missPLS", envir = environment())
  data("octane", package = "missPLS", envir = environment())
  data("ozone_complete", package = "missPLS", envir = environment())

  expect_s3_class(bromhexine, "misspls_dataset")
  expect_identical(dim(bromhexine$x), c(23L, 64L))
  expect_length(bromhexine$y, 23L)

  expect_s3_class(tetracycline, "misspls_dataset")
  expect_identical(dim(tetracycline$x), c(107L, 101L))
  expect_length(tetracycline$y, 107L)

  expect_s3_class(octane, "misspls_dataset")
  expect_identical(dim(octane$x), c(68L, 493L))
  expect_length(octane$y, 68L)

  expect_s3_class(ozone_complete, "misspls_dataset")
  expect_identical(dim(ozone_complete$x), c(203L, 12L))
  expect_length(ozone_complete$y, 203L)
})

---
title: "missPLS"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{missPLS}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

`missPLS` provides methods-first tooling for the PLS workflows described in the
article and thesis on incomplete-data NIPALS-PLS regression.

## Simulate a dataset

```{r}
library(missPLS)

sim <- simulate_pls_data(n = 30, p = 12, true_ncomp = 2, seed = 1)
str(sim, max.level = 1)
```

## Add missingness and select components

```{r}
miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)

select_ncomp(
  x = miss$x_incomplete,
  y = sim$y,
  method = "nipals_standard",
  criterion = "Q2-LOO",
  max_ncomp = 4,
  seed = 3
)
```

## Impute and re-fit

```{r}
imp <- impute_pls_data(miss$x_incomplete, method = "knn", seed = 4)

select_ncomp(
  x = imp,
  y = sim$y,
  method = "complete",
  criterion = "Q2-10fold",
  max_ncomp = 4,
  seed = 5,
  folds = 5
)
```

## Real-data diagnostics

```{r}
diag_bromhexine <- diagnose_real_data("bromhexine")
head(diag_bromhexine$predictor_correlations)
head(diag_bromhexine$response_correlations)
```

## Study runners

The study runners are intended to orchestrate smoke runs in-package and heavier
reproduction runs under `tools/`.

```{r}
results <- run_simulation_study(
  dimensions = list(c(25, 10)),
  true_ncomp = 2,
  missing_props = 10,
  mechanisms = "MCAR",
  reps = 2,
  seed = 1,
  max_ncomp = 4,
  criteria = c("Q2-LOO", "AIC"),
  incomplete_methods = "nipals_standard",
  imputation_methods = "knn",
  folds = 5
)

summarize_simulation_study(results)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_simulation_study.R
\name{run_simulation_study}
\alias{run_simulation_study}
\title{Run a simulation study}
\usage{
run_simulation_study(
  dimensions = list(c(500L, 100L), c(500L, 20L), c(100L, 20L), c(80L, 25L), c(60L, 33L),
    c(40L, 50L), c(20L, 100L)),
  true_ncomp = c(2L, 4L, 6L),
  missing_props = seq(5, 50, 5),
  mechanisms = c("MCAR", "MAR"),
  reps = 1L,
  seed = NULL,
  max_ncomp = 8L,
  criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
  incomplete_methods = c("nipals_standard", "nipals_adaptative"),
  imputation_methods = c("mice", "knn", "svd"),
  folds = 10L,
  mar_y_bias = 0.8
)
}
\arguments{
\item{dimensions}{List of \verb{(n, p)} integer pairs.}

\item{true_ncomp}{Vector of true component counts.}

\item{missing_props}{Missing-data proportions as fractions or percentages.}

\item{mechanisms}{Missing-data mechanisms.}

\item{reps}{Number of replicates.}

\item{seed}{Optional base random seed.}

\item{max_ncomp}{Maximum number of extracted components.}

\item{criteria}{Criteria evaluated on complete and imputed data.}

\item{incomplete_methods}{Incomplete-data NIPALS workflows.}

\item{imputation_methods}{Imputation methods.}

\item{folds}{Number of folds used by \code{"Q2-10fold"}.}

\item{mar_y_bias}{MAR bias parameter passed to \code{\link[=add_missingness]{add_missingness()}}.}
}
\value{
A data frame with one row per study run.
}
\description{
Run the simulation workflows used in the article and thesis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_pls_data.R
\name{simulate_pls_data}
\alias{simulate_pls_data}
\title{Simulate PLS data}
\usage{
simulate_pls_data(n, p, true_ncomp, seed = NULL, model = "li2002")
}
\arguments{
\item{n}{Number of observations.}

\item{p}{Number of predictors.}

\item{true_ncomp}{True number of latent components.}

\item{seed}{Optional random seed. If supplied, it is used only for this call.}

\item{model}{Simulation model. Only \code{"li2002"} is currently supported.}
}
\value{
A list with components \code{x}, \code{y}, \code{data}, \code{true_ncomp}, \code{seed}, and
\code{model}.
}
\description{
Simulate a univariate-response PLS dataset using the Li et al.-style
generator available in \code{plsRglm}.
}
\examples{
sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 42)
str(sim)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_simulation_study.R
\name{summarize_simulation_study}
\alias{summarize_simulation_study}
\title{Summarize simulation or real-data study results}
\usage{
summarize_simulation_study(results)
}
\arguments{
\item{results}{A results data frame returned by \code{\link[=run_simulation_study]{run_simulation_study()}} or
\code{\link[=run_real_data_study]{run_real_data_study()}}.}
}
\value{
A grouped summary data frame.
}
\description{
Summarize simulation or real-data study results
}
\examples{
sim_results <- run_simulation_study(
  dimensions = list(c(30, 12)),
  true_ncomp = 2,
  missing_props = numeric(0),
  mechanisms = character(0),
  reps = 2,
  seed = 1
)
summarize_simulation_study(sim_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_pls_data.R
\name{impute_pls_data}
\alias{impute_pls_data}
\title{Impute a predictor matrix}
\usage{
impute_pls_data(
  x,
  method = c("mice", "knn", "svd"),
  seed = NULL,
  m,
  k = 15L,
  svd_rank = 10L,
  svd_maxiter = 1000L
)
}
\arguments{
\item{x}{Incomplete predictor matrix or data frame.}

\item{method}{Imputation method: \code{"mice"}, \code{"knn"}, or \code{"svd"}.}

\item{seed}{Optional random seed forwarded to stochastic imputers when
supported.}

\item{m}{Number of imputations for \code{method = "mice"}. By default this is set
to the missing-data percentage rounded to the nearest integer.}

\item{k}{Number of neighbours for \code{method = "knn"}.}

\item{svd_rank}{Target rank for \code{method = "svd"}.}

\item{svd_maxiter}{Maximum number of iterations for the fallback SVD routine.}
}
\value{
A \code{misspls_imputation} object.
}
\description{
Apply one of the imputation strategies used in the article and thesis.
}
\examples{
sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 1)
miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
imp <- impute_pls_data(miss$x_incomplete, method = "knn", seed = 3)
length(imp$datasets)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{missPLS-package}
\alias{missPLS}
\alias{missPLS-package}
\title{missPLS: Methods and Reproducible Workflows for Partial Least Squares with Missing Data}
\description{
Methods-first tooling for reproducing and extending the partial least squares regression studies on incomplete data described in Nengsih et al. (2019) \doi{10.1515/sagmb-2018-0059}. The package provides simulation helpers, missingness generators, imputation wrappers, component-selection utilities, real-data diagnostics, and reproducible study orchestration for Nonlinear Iterative Partial Least Squares (NIPALS)-Partial Least Squares (PLS) workflows.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://fbertran.github.io/missPLS/}
  \item \url{https://github.com/fbertran/missPLS}
  \item Report bugs at \url{https://github.com/fbertran/missPLS/issues}
}

}
\author{
\strong{Maintainer}: Frederic Bertrand \email{frederic.bertrand@lecnam.net}

Authors:
\itemize{
  \item Titin Agustin Nengsih
  \item Myriam Maumy-Bertrand
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{bromhexine}
\alias{bromhexine}
\title{Bromhexine dataset}
\format{
A \code{misspls_dataset} list with components:
\describe{
\item{name}{Dataset name.}
\item{x}{A numeric \verb{23 x 64} predictor matrix.}
\item{y}{A numeric response vector of length \code{23}.}
\item{data}{A data frame with response \code{y} and predictors \code{x1} to \code{x64}.}
\item{source}{A short source reference.}
\item{preprocessing}{Dataset preprocessing notes.}
\item{notes}{Additional study notes.}
}
}
\source{
Goicoechea and Olivieri (1999a), calibration and test files bundled in
\code{extra_docs/pls_data}.
}
\usage{
bromhexine
}
\description{
Bromhexine in pharmaceutical syrup used in the article and thesis.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_real_data_study.R
\name{run_real_data_study}
\alias{run_real_data_study}
\title{Run a real-data study}
\usage{
run_real_data_study(
  dataset,
  seed = NULL,
  missing_props = seq(5, 50, 5),
  mechanisms = c("MCAR", "MAR"),
  reps = 1L,
  baseline_reps = 100L,
  max_ncomp = 12L,
  criteria = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
  incomplete_methods = c("nipals_standard", "nipals_adaptative"),
  imputation_methods = c("mice", "knn", "svd"),
  folds = 10L,
  mar_y_bias = 0.8
)
}
\arguments{
\item{dataset}{A packaged dataset name or \code{misspls_dataset} object.}

\item{seed}{Optional base random seed.}

\item{missing_props}{Missing-data proportions as fractions or percentages.}

\item{mechanisms}{Missing-data mechanisms.}

\item{reps}{Number of replicate missingness draws for each mechanism and
proportion.}

\item{baseline_reps}{Number of repeated complete-data \verb{Q2-10fold} selections
used to determine \verb{t**}.}

\item{max_ncomp}{Maximum number of extracted components.}

\item{criteria}{Criteria evaluated on incomplete and imputed data.}

\item{incomplete_methods}{Incomplete-data NIPALS workflows.}

\item{imputation_methods}{Imputation methods.}

\item{folds}{Number of folds used by \code{"Q2-10fold"}.}

\item{mar_y_bias}{MAR bias parameter passed to \code{\link[=add_missingness]{add_missingness()}}.}
}
\value{
A data frame with one row per study run.
}
\description{
Run a real-data study
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_missingness.R
\name{add_missingness}
\alias{add_missingness}
\title{Add missing values to a predictor matrix}
\usage{
add_missingness(
  x,
  y,
  mechanism = c("MCAR", "MAR"),
  missing_prop,
  seed = NULL,
  mar_y_bias = 0.8
)
}
\arguments{
\item{x}{Predictor matrix or data frame.}

\item{y}{Numeric response vector.}

\item{mechanism}{Missingness mechanism: \code{"MCAR"} or \code{"MAR"}.}

\item{missing_prop}{Missing-data proportion as a fraction (\code{0.05}) or a
percentage (\code{5}).}

\item{seed}{Optional random seed. If supplied, it is used only for this call.}

\item{mar_y_bias}{Proportion of missing values assigned to the upper
half of the observed \code{y} values under the MAR mechanism.}
}
\value{
A list with components \code{x_incomplete}, \code{missing_mask},
\code{missing_prop}, \code{mechanism}, and \code{seed}.
}
\description{
Create MCAR or MAR missingness on the predictor matrix \code{x}. Missingness is
generated column-wise so that each predictor receives approximately the same
missing-data proportion, matching the simulation strategy used in the
original work.
}
\examples{
sim <- simulate_pls_data(n = 20, p = 10, true_ncomp = 2, seed = 1)
miss <- add_missingness(sim$x, sim$y, mechanism = "MCAR", missing_prop = 10, seed = 2)
mean(is.na(miss$x_incomplete))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{ozone_complete}
\alias{ozone_complete}
\title{Complete-case ozone dataset}
\format{
A \code{misspls_dataset} list with components:
\describe{
\item{name}{Dataset name.}
\item{x}{A numeric \verb{203 x 12} predictor matrix.}
\item{y}{A numeric response vector of length \code{203}.}
\item{data}{A data frame with response \code{y} and predictors \code{x1} to \code{x12}.}
\item{source}{A short source reference.}
\item{preprocessing}{Dataset preprocessing notes.}
\item{notes}{Additional study notes.}
}
}
\source{
\code{mlbench::Ozone}, restricted to the \code{203} complete observations used
in the published analysis.
}
\usage{
ozone_complete
}
\description{
Los Angeles ozone pollution complete-case dataset used in the article and
thesis.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose_real_data.R
\name{diagnose_real_data}
\alias{diagnose_real_data}
\title{Diagnose a real dataset}
\usage{
diagnose_real_data(dataset, cor_threshold = 0.7)
}
\arguments{
\item{dataset}{A packaged dataset name or \code{misspls_dataset} object.}

\item{cor_threshold}{Absolute-correlation threshold used when reporting
predictor pairs and predictor-response associations.}
}
\value{
A list with correlation and VIF summaries.
}
\description{
Compute correlation summaries and VIF-style diagnostics for a packaged
real dataset.
}
\examples{
diag_bromhexine <- diagnose_real_data("bromhexine")
names(diag_bromhexine)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_ncomp.R
\name{select_ncomp}
\alias{select_ncomp}
\title{Select the number of PLS components}
\usage{
select_ncomp(
  x,
  y,
  method = c("complete", "nipals_standard", "nipals_adaptative"),
  criterion = c("Q2-LOO", "Q2-10fold", "AIC", "AIC-DoF", "BIC", "BIC-DoF"),
  max_ncomp,
  seed = NULL,
  folds = 10L,
  threshold = 0.0975
)
}
\arguments{
\item{x}{Predictor matrix, dataset object, or \code{misspls_imputation} object.}

\item{y}{Numeric response vector. This may be omitted when \code{x} already
contains a response.}

\item{method}{Selection workflow: \code{"complete"}, \code{"nipals_standard"}, or
\code{"nipals_adaptative"}.}

\item{criterion}{Selection criterion: \code{"Q2-LOO"}, \code{"Q2-10fold"}, \code{"AIC"},
\code{"AIC-DoF"}, \code{"BIC"}, or \code{"BIC-DoF"}.}

\item{max_ncomp}{Maximum number of components to consider.}

\item{seed}{Optional random seed used by the cross-validation and imputation
aggregation steps.}

\item{folds}{Number of cross-validation folds used by \code{"Q2-10fold"}.}

\item{threshold}{Threshold applied to \code{Q2} criteria.}
}
\value{
A one-row data frame describing the selected component count.
}
\description{
Select the number of components for complete, imputed, or incomplete-data
PLS workflows.
}
\examples{
sim <- simulate_pls_data(n = 25, p = 10, true_ncomp = 2, seed = 1)
select_ncomp(sim$x, sim$y, method = "complete", criterion = "AIC", max_ncomp = 4, seed = 2)
}
